import events = require('@aws-cdk/aws-events');
import iam = require('@aws-cdk/aws-iam');
import s3 = require('@aws-cdk/aws-s3');
import { Construct, Resource } from '@aws-cdk/cdk';
import { Action, IPipeline, IStage } from "./action";
import { CrossRegionScaffoldStack } from './cross-region-scaffold-stack';
/**
 * Allows you to control where to place a new Stage when it's added to the Pipeline.
 * Note that you can provide only one of the below properties -
 * specifying more than one will result in a validation error.
 *
 * @see #rightBefore
 * @see #justAfter
 * @see #atIndex
 */
export interface StagePlacement {
    /**
     * Inserts the new Stage as a parent of the given Stage
     * (changing its current parent Stage, if it had one).
     */
    readonly rightBefore?: IStage;
    /**
     * Inserts the new Stage as a child of the given Stage
     * (changing its current child Stage, if it had one).
     */
    readonly justAfter?: IStage;
    /**
     * Inserts the new Stage at the given index in the Pipeline,
     * moving the Stage currently at that index,
     * and any subsequent ones, one index down.
     * Indexing starts at 0.
     * The maximum allowed value is {@link Pipeline#stageCount},
     * which will insert the new Stage at the end of the Pipeline.
     */
    readonly atIndex?: number;
}
/**
 * Construction properties of a Pipeline Stage.
 */
export interface StageProps {
    /**
     * The physical, human-readable name to assign to this Pipeline Stage.
     */
    readonly name: string;
    /**
     * The list of Actions to create this Stage with.
     * You can always add more Actions later by calling {@link IStage#addAction}.
     */
    readonly actions?: Action[];
}
export interface StageAddToPipelineProps extends StageProps {
    readonly placement?: StagePlacement;
}
export interface PipelineProps {
    /**
     * The S3 bucket used by this Pipeline to store artifacts.
     * If not specified, a new S3 bucket will be created.
     */
    readonly artifactBucket?: s3.IBucket;
    /**
     * Indicates whether to rerun the AWS CodePipeline pipeline after you update it.
     */
    readonly restartExecutionOnUpdate?: boolean;
    /**
     * Name of the pipeline. If you don't specify a name,  AWS CloudFormation generates an ID
     * and uses that for the pipeline name.
     */
    readonly pipelineName?: string;
    /**
     * A map of region to S3 bucket name used for cross-region CodePipeline.
     * For every Action that you specify targeting a different region than the Pipeline itself,
     * if you don't provide an explicit Bucket for that region using this property,
     * the construct will automatically create a scaffold Stack containing an S3 Bucket in that region.
     * Note that you will have to `cdk deploy` that Stack before you can deploy your Pipeline-containing Stack.
     * You can query the generated Stacks using the {@link Pipeline#crossRegionScaffoldStacks} property.
     */
    readonly crossRegionReplicationBuckets?: {
        [region: string]: string;
    };
    /**
     * The list of Stages, in order,
     * to create this Pipeline with.
     * You can always add more Stages later by calling {@link Pipeline#addStage}.
     */
    readonly stages?: StageProps[];
}
declare abstract class PipelineBase extends Resource implements IPipeline {
    abstract pipelineName: string;
    abstract pipelineArn: string;
    private eventsRole?;
    abstract grantBucketRead(identity: iam.IGrantable): iam.Grant;
    abstract grantBucketReadWrite(identity: iam.IGrantable): iam.Grant;
    /**
     * Allows the pipeline to be used as a CloudWatch event rule target.
     *
     * Usage:
     *
     *    const pipeline = new Pipeline(this, 'MyPipeline');
     *    const rule = new EventRule(this, 'MyRule', { schedule: 'rate(1 minute)' });
     *    rule.addTarget(pipeline);
     *
     */
    asEventRuleTarget(_ruleArn: string, _ruleId: string): events.EventRuleTargetProps;
}
/**
 * An AWS CodePipeline pipeline with its associated IAM role and S3 bucket.
 *
 * @example
 * // create a pipeline
 * const pipeline = new Pipeline(this, 'Pipeline');
 *
 * // add a stage
 * const sourceStage = pipeline.addStage({ name: 'Source' });
 *
 * // add a source action to the stage
 * sourceStage.addAction(new codepipeline_actions.CodeCommitSourceAction({
 *   actionName: 'Source',
 *   outputArtifactName: 'SourceArtifact',
 *   repository: repo,
 * }));
 *
 * // ... add more stages
 */
export declare class Pipeline extends PipelineBase {
    /**
     * Import a pipeline into this app.
     * @param scope the scope into which to import this pipeline
     * @param pipelineArn The ARN of the pipeline (e.g. `arn:aws:codepipeline:us-east-1:123456789012:MyDemoPipeline`)
     */
    static fromPipelineArn(scope: Construct, id: string, pipelineArn: string): IPipeline;
    /**
     * The IAM role AWS CodePipeline will use to perform actions or assume roles for actions with
     * a more specific IAM role.
     */
    readonly role: iam.Role;
    /**
     * ARN of this pipeline
     */
    readonly pipelineArn: string;
    /**
     * The name of the pipeline
     */
    readonly pipelineName: string;
    /**
     * The version of the pipeline
     *
     * @attribute
     */
    readonly pipelineVersion: string;
    /**
     * Bucket used to store output artifacts
     */
    readonly artifactBucket: s3.IBucket;
    private readonly stages;
    private readonly pipelineResource;
    private readonly crossRegionReplicationBuckets;
    private readonly artifactStores;
    private readonly _crossRegionScaffoldStacks;
    constructor(scope: Construct, id: string, props?: PipelineProps);
    /**
     * Creates a new Stage, and adds it to this Pipeline.
     *
     * @param props the creation properties of the new Stage
     * @returns the newly created Stage
     */
    addStage(props: StageAddToPipelineProps): IStage;
    /**
     * Adds a statement to the pipeline role.
     */
    addToRolePolicy(statement: iam.PolicyStatement): void;
    /**
     * Defines an event rule triggered by the "CodePipeline Pipeline Execution
     * State Change" event emitted from this pipeline.
     *
     * @param target Initial target to add to the event rule. You can also add
     * targets and customize target inputs by calling `rule.addTarget(target[,
     * options])` after the rule was created.
     *
     * @param options Additional options to pass to the event rule
     *
     * @param name The name of the event rule construct. If you wish to define
     * more than a single onStateChange event, you will need to explicitly
     * specify a name.
     */
    onStateChange(name: string, target?: events.IEventRuleTarget, options?: events.EventRuleProps): events.EventRule;
    /**
     * Get the number of Stages in this Pipeline.
     */
    readonly stageCount: number;
    grantBucketRead(identity: iam.IGrantable): iam.Grant;
    grantBucketReadWrite(identity: iam.IGrantable): iam.Grant;
    /**
     * Returns all of the {@link CrossRegionScaffoldStack}s that were generated automatically
     * when dealing with Actions that reside in a different region than the Pipeline itself.
     */
    readonly crossRegionScaffoldStacks: {
        [region: string]: CrossRegionScaffoldStack;
    };
    /**
     * Validate the pipeline structure
     *
     * Validation happens according to the rules documented at
     *
     * https://docs.aws.amazon.com/codepipeline/latest/userguide/reference-pipeline-structure.html#pipeline-requirements
     * @override
     */
    protected validate(): string[];
    private _attachActionToPipeline;
    private ensureReplicationBucketExistsFor;
    private calculateInsertIndexFromPlacement;
    private findStageIndex;
    private validateSourceActionLocations;
    private validateHasStages;
    private validateStages;
    private validateArtifacts;
    private renderArtifactStore;
    private renderStages;
}
export {};
